@@ -30,14 +30,21 @@ __all__ = [
     "_calc_memory_bytes_LPRec3d_tomobar",
     "_calc_memory_bytes_SIRT3d_tomobar",
     "_calc_memory_bytes_CGLS3d_tomobar",
+    "_calc_memory_bytes_FISTA3d_tomobar",
     "_calc_output_dim_FBP2d_astra",
     "_calc_output_dim_FBP3d_tomobar",
     "_calc_output_dim_LPRec3d_tomobar",
     "_calc_output_dim_SIRT3d_tomobar",
     "_calc_output_dim_CGLS3d_tomobar",
+    "_calc_output_dim_FISTA3d_tomobar",
+    "_calc_padding_FISTA3d_tomobar",
 ]
 
 
+def _calc_padding_FISTA3d_tomobar(**kwargs) -> Tuple[int, int]:
+    return (5, 5)
+
+
 def __calc_output_dim_recon(non_slice_dims_shape, **kwargs):
     """Function to calculate output dimensions for all reconstructors.
     The change of the dimension depends either on the user-provided "recon_size"
@@ -72,14 +79,21 @@ def _calc_output_dim_CGLS3d_tomobar(non_slice_dims_shape, **kwargs):
     return __calc_output_dim_recon(non_slice_dims_shape, **kwargs)
 
 
+def _calc_output_dim_FISTA3d_tomobar(non_slice_dims_shape, **kwargs):
+    return __calc_output_dim_recon(non_slice_dims_shape, **kwargs)
+
+
 def _calc_memory_bytes_FBP3d_tomobar(
     non_slice_dims_shape: Tuple[int, int],
     dtype: np.dtype,
     **kwargs,
 ) -> Tuple[int, int]:
+    detector_pad = 0
     if "detector_pad" in kwargs:
         detector_pad = kwargs["detector_pad"]
-    else:
+    if detector_pad is True:
+        detector_pad = __estimate_detectorHoriz_padding(non_slice_dims_shape[1])
+    elif detector_pad is False:
         detector_pad = 0
 
     angles_tot = non_slice_dims_shape[0]
@@ -132,11 +146,11 @@ def _calc_memory_bytes_FBP3d_tomobar(
 
     # 7. astra backprojection will generate an output array
     # https://github.com/dkazanc/ToMoBAR/blob/54137829b6326406e09f6ef9c95eb35c213838a7/tomobar/astra_wrappers/astra_base.py#L524
-    output_dims = _calc_output_dim_FBP3d_tomobar(non_slice_dims_shape, **kwargs)
+    output_dims = _calc_output_dim_FBP3d_tomobar((angles_tot, det_width), **kwargs)
     recon_output_size = np.prod(output_dims) * np.float32().itemsize
 
     # 7. astra backprojection makes a copy of the input
-    astra_input_slice_size = np.prod(non_slice_dims_shape) * np.float32().itemsize
+    astra_input_slice_size = (angles_tot * det_width) * np.float32().itemsize
 
     ## now we calculate back projection memory (2 copies of the input + reconstruction output)
     projection_mem_size = (
@@ -169,10 +183,16 @@ def _calc_memory_bytes_LPRec3d_tomobar(
 ) -> Tuple[int, int]:
     # Based on: https://github.com/dkazanc/ToMoBAR/pull/112/commits/4704ecdc6ded3dd5ec0583c2008aa104f30a8a39
 
+    detector_pad = 0
     if "detector_pad" in kwargs:
         detector_pad = kwargs["detector_pad"]
-    else:
-        detector_pad = 0
+    if detector_pad is True:
+        detector_pad = __estimate_detectorHoriz_padding(non_slice_dims_shape[1])
+    elif detector_pad is False:
+        detector_pad = 0        
+
+    min_mem_usage_filter = False
+    min_mem_usage_ifft2 = False
 
     angles_tot = non_slice_dims_shape[0]
     DetectorsLengthH_prepad = non_slice_dims_shape[1]
@@ -195,11 +215,22 @@ def _calc_memory_bytes_LPRec3d_tomobar(
         )
     )
 
-    center_size = 6144
-    center_size = min(center_size, n * 2 + m * 2)
+    center_size = 32768
+    center_size = min(center_size, n * 2)
+
+    chunk_count = 4
+    projection_chunk_count = 4
+    oversampling_level = 4  # at least 3 or larger required
+    power_of_2_oversampling = True
+
+    if power_of_2_oversampling:
+        ne = 2 ** math.ceil(math.log2(DetectorsLengthH_prepad * 3))
+        if n > ne:
+            ne = 2 ** math.ceil(math.log2(n))
+    else:
+        ne = int(oversampling_level * DetectorsLengthH_prepad)
+        ne = max(ne, n)
 
-    oversampling_level = 2  # at least 2 or larger required
-    ne = oversampling_level * n
     padding_m = ne // 2 - n // 2
 
     if "angles" in kwargs:
@@ -213,8 +244,6 @@ def _calc_memory_bytes_LPRec3d_tomobar(
             np.ceil(2)
         )  # assume a 2 * PI projection angle range
 
-    chunk_count = 4
-
     output_dims = __calc_output_dim_recon(non_slice_dims_shape, **kwargs)
     if odd_horiz:
         output_dims = tuple(x + 1 for x in output_dims)
@@ -252,7 +281,7 @@ def _calc_memory_bytes_LPRec3d_tomobar(
     irfft_result_size = angles_tot * (n + padding_m * 2) * np.float32().itemsize
 
     datac_size = angles_tot * n * np.complex64().itemsize / 2
-    fde_size = (2 * m + 2 * n) * (2 * m + 2 * n) * np.complex64().itemsize / 2
+    fde_size = 2 * n * 2 * n * np.complex64().itemsize / 2
     fft_plan_slice_size = (
         cufft_estimate_1d(nx=n, fft_type=CufftType.CUFFT_C2C, batch=angles_tot * SLICES)
         / SLICES
@@ -270,7 +299,7 @@ def _calc_memory_bytes_LPRec3d_tomobar(
     )
     ifft2_plan_slice_size = (
         cufft_estimate_2d(
-            nx=(2 * m + 2 * n), ny=(2 * m + 2 * n), fft_type=CufftType.CUFFT_C2C
+            nx=2 * n, ny=2 * n, fft_type=CufftType.CUFFT_C2C
         )
         / 2
     )
@@ -309,24 +338,40 @@ def _calc_memory_bytes_LPRec3d_tomobar(
     add_to_memory_counters(scaled_filter_size, False)
 
     add_to_memory_counters(tmp_p_input_slice, True)
+    if min_mem_usage_filter:
+        add_to_memory_counters(rfft_plan_slice_size / 4, False)
+        add_to_memory_counters(irfft_plan_slice_size / 4, False)
+        add_to_memory_counters(padded_tmp_p_input_slice, False)
 
-    add_to_memory_counters(rfft_plan_slice_size / chunk_count * 2, True)
-    add_to_memory_counters(irfft_plan_slice_size / chunk_count * 2, True)
-    # add_to_memory_counters(irfft_scratch_memory_size / chunk_count, True)
-    for _ in range(0, chunk_count):
-        add_to_memory_counters(padded_tmp_p_input_slice / chunk_count, True)
+        add_to_memory_counters(rfft_result_size, False)
+        add_to_memory_counters(filtered_rfft_result_size, False)
+        add_to_memory_counters(-rfft_result_size, False)
+        add_to_memory_counters(-padded_tmp_p_input_slice, False)
 
-        add_to_memory_counters(rfft_result_size / chunk_count, True)
-        add_to_memory_counters(filtered_rfft_result_size / chunk_count, True)
-        add_to_memory_counters(-rfft_result_size / chunk_count, True)
-        add_to_memory_counters(-padded_tmp_p_input_slice / chunk_count, True)
+        add_to_memory_counters(irfft_scratch_memory_size, False)
+        add_to_memory_counters(-irfft_scratch_memory_size, False)
+        add_to_memory_counters(irfft_result_size, False)
+        add_to_memory_counters(-filtered_rfft_result_size, False)
 
-        add_to_memory_counters(irfft_scratch_memory_size / chunk_count, True)
-        add_to_memory_counters(-irfft_scratch_memory_size / chunk_count, True)
-        add_to_memory_counters(irfft_result_size / chunk_count, True)
-        add_to_memory_counters(-filtered_rfft_result_size / chunk_count, True)
+        add_to_memory_counters(-irfft_result_size, False)
+    else:
+        add_to_memory_counters(rfft_plan_slice_size / chunk_count / projection_chunk_count * 2, True)
+        add_to_memory_counters(irfft_plan_slice_size / chunk_count / projection_chunk_count * 2, True)
+        # add_to_memory_counters(irfft_scratch_memory_size / chunk_count / projection_chunk_count, True)
+        for _ in range(0, chunk_count):
+            add_to_memory_counters(padded_tmp_p_input_slice / chunk_count / projection_chunk_count, True)
 
-        add_to_memory_counters(-irfft_result_size / chunk_count, True)
+            add_to_memory_counters(rfft_result_size / chunk_count / projection_chunk_count, True)
+            add_to_memory_counters(filtered_rfft_result_size / chunk_count / projection_chunk_count, True)
+            add_to_memory_counters(-rfft_result_size / chunk_count / projection_chunk_count, True)
+            add_to_memory_counters(-padded_tmp_p_input_slice / chunk_count / projection_chunk_count, True)
+
+            add_to_memory_counters(irfft_scratch_memory_size / chunk_count / projection_chunk_count, True)
+            add_to_memory_counters(-irfft_scratch_memory_size / chunk_count / projection_chunk_count, True)
+            add_to_memory_counters(irfft_result_size / chunk_count / projection_chunk_count, True)
+            add_to_memory_counters(-filtered_rfft_result_size / chunk_count / projection_chunk_count, True)
+
+            add_to_memory_counters(-irfft_result_size / chunk_count / projection_chunk_count, True)
 
     add_to_memory_counters(-padded_in_slice_size, True)
     add_to_memory_counters(-filter_size, False)
@@ -342,17 +387,25 @@ def _calc_memory_bytes_LPRec3d_tomobar(
 
     add_to_memory_counters(-fft_result_size, True)
 
-    add_to_memory_counters(ifft2_plan_slice_size / chunk_count * 2, True)
-    for _ in range(0, chunk_count):
-        add_to_memory_counters(fde_size / chunk_count, True)
-        add_to_memory_counters(-fde_size / chunk_count, True)
+    if min_mem_usage_ifft2:
+        add_to_memory_counters(ifft2_plan_slice_size, False)
+        add_to_memory_counters(fde_size * 2, False)
+        add_to_memory_counters(-fde_size * 2, False)
+    else:
+        add_to_memory_counters(ifft2_plan_slice_size / chunk_count * 2, True)
+        for _ in range(0, chunk_count):
+            add_to_memory_counters(fde_size / chunk_count, True)
+            add_to_memory_counters(-fde_size / chunk_count, True)
 
     add_to_memory_counters(recon_output_size, True)
     add_to_memory_counters(-fde_size, True)
     add_to_memory_counters(circular_mask_size, False)
     add_to_memory_counters(after_recon_swapaxis_slice, True)
 
-    return (tot_memory_bytes * 1.05, fixed_amount + 250 * 1024 * 1024)
+    if min_mem_usage_ifft2 and min_mem_usage_filter:
+        return (tot_memory_bytes * 1.1 + 30 * 1024 * 1024, fixed_amount)
+    else:
+        return (tot_memory_bytes * 1.1, fixed_amount)
 
 
 def _calc_memory_bytes_SIRT3d_tomobar(
@@ -361,21 +414,48 @@ def _calc_memory_bytes_SIRT3d_tomobar(
     **kwargs,
 ) -> Tuple[int, int]:
 
+    detector_pad = 0
     if "detector_pad" in kwargs:
         detector_pad = kwargs["detector_pad"]
-    else:
-        detector_pad = 0
+    if detector_pad is True:
+        detector_pad = __estimate_detectorHoriz_padding(non_slice_dims_shape[1])
+    elif detector_pad is False:
+        detector_pad = 0        
+
     anglesnum = non_slice_dims_shape[0]
-    DetectorsLengthH = non_slice_dims_shape[1] + 2 * detector_pad
+    DetectorsLengthH_padded = non_slice_dims_shape[1] + 2 * detector_pad
     # calculate the output shape
     output_dims = _calc_output_dim_SIRT3d_tomobar(non_slice_dims_shape, **kwargs)
+    recon_data_size_original = (
+        np.prod(output_dims) * dtype.itemsize
+    )  # x_rec user-defined size
 
-    in_data_size = (anglesnum * DetectorsLengthH) * dtype.itemsize
-    out_data_size = np.prod(output_dims) * dtype.itemsize
+    in_data_size = (anglesnum * DetectorsLengthH_padded) * dtype.itemsize
 
-    astra_projection = 2.5 * (in_data_size + out_data_size)
+    output_dims_larger_grid = (DetectorsLengthH_padded, DetectorsLengthH_padded)
 
-    tot_memory_bytes = int(2 * in_data_size + 2 * out_data_size + astra_projection)
+    out_data_size = np.prod(output_dims_larger_grid) * dtype.itemsize
+
+    R = in_data_size
+    C = out_data_size
+
+    Res = in_data_size
+    Res_times_R = Res
+    C_times_res = out_data_size
+
+    astra_projection = (in_data_size + out_data_size)
+
+    tot_memory_bytes = int(
+        recon_data_size_original
+        + in_data_size
+        + out_data_size
+        + R
+        + C
+        + Res
+        + Res_times_R
+        + C_times_res
+        + astra_projection
+    )
     return (tot_memory_bytes, 0)
 
 
@@ -384,20 +464,102 @@ def _calc_memory_bytes_CGLS3d_tomobar(
     dtype: np.dtype,
     **kwargs,
 ) -> Tuple[int, int]:
+    detector_pad = 0
     if "detector_pad" in kwargs:
         detector_pad = kwargs["detector_pad"]
-    else:
-        detector_pad = 0
+    if detector_pad is True:
+        detector_pad = __estimate_detectorHoriz_padding(non_slice_dims_shape[1])
+    elif detector_pad is False:
+        detector_pad = 0        
 
     anglesnum = non_slice_dims_shape[0]
-    DetectorsLengthH = non_slice_dims_shape[1] + 2 * detector_pad
+    DetectorsLengthH_padded = non_slice_dims_shape[1] + 2 * detector_pad
     # calculate the output shape
     output_dims = _calc_output_dim_CGLS3d_tomobar(non_slice_dims_shape, **kwargs)
+    recon_data_size_original = (
+        np.prod(output_dims) * dtype.itemsize
+    )  # x_rec user-defined size
 
-    in_data_size = (anglesnum * DetectorsLengthH) * dtype.itemsize
-    out_data_size = np.prod(output_dims) * dtype.itemsize
+    in_data_size = (anglesnum * DetectorsLengthH_padded) * dtype.itemsize
+    output_dims_larger_grid = (DetectorsLengthH_padded, DetectorsLengthH_padded)
+    recon_data_size = (
+        np.prod(output_dims_larger_grid) * dtype.itemsize
+    )  # large volume in the algorithm
+    recon_data_size2 = recon_data_size  # x_rec linearised
+    d_recon = recon_data_size
+    d_recon2 = d_recon  # linearised, possibly a copy
 
-    astra_projection = 2.5 * (in_data_size + out_data_size)
+    data_r = in_data_size
+    Ad = recon_data_size
+    Ad2 = Ad
+    s = data_r
+    collection = (
+        in_data_size
+        + recon_data_size_original
+        + recon_data_size
+        + recon_data_size2
+        + d_recon
+        + d_recon2
+        + data_r
+        + Ad
+        + Ad2
+        + s
+    )
+    astra_contribution = in_data_size + recon_data_size
 
-    tot_memory_bytes = int(2 * in_data_size + 2 * out_data_size + astra_projection)
+    tot_memory_bytes = int(collection + astra_contribution)
     return (tot_memory_bytes, 0)
+
+
+def _calc_memory_bytes_FISTA3d_tomobar(
+    non_slice_dims_shape: Tuple[int, int],
+    dtype: np.dtype,
+    **kwargs,
+) -> Tuple[int, int]:
+    detector_pad = 0
+    if "detector_pad" in kwargs:
+        detector_pad = kwargs["detector_pad"]
+    if detector_pad is True:
+        detector_pad = __estimate_detectorHoriz_padding(non_slice_dims_shape[1])
+    elif detector_pad is False:
+        detector_pad = 0        
+
+    anglesnum = non_slice_dims_shape[0]
+    DetectorsLengthH_padded = non_slice_dims_shape[1] + 2 * detector_pad
+
+    # calculate the output shape
+    output_dims = _calc_output_dim_FISTA3d_tomobar(non_slice_dims_shape, **kwargs)
+    recon_data_size_original = (
+        np.prod(output_dims) * dtype.itemsize
+    )  # recon user-defined size
+
+    in_data_siz_pad = (anglesnum * DetectorsLengthH_padded) * dtype.itemsize
+    output_dims_larger_grid = (DetectorsLengthH_padded, DetectorsLengthH_padded)
+
+    residual_grad = in_data_siz_pad
+    out_data_size = np.prod(output_dims_larger_grid) * dtype.itemsize
+    X_t = out_data_size
+    X_old = out_data_size
+
+    grad_fidelity = out_data_size
+
+    fista_part = (
+        recon_data_size_original
+        + in_data_siz_pad
+        + residual_grad
+        + grad_fidelity
+        + X_t
+        + X_old
+        + out_data_size
+    )
+    regul_part = 8 * np.prod(output_dims_larger_grid) * dtype.itemsize
+
+    tot_memory_bytes = int(fista_part + regul_part)
+    return (tot_memory_bytes, 0)
+
+
+def __estimate_detectorHoriz_padding(detX_size) -> int:
+    det_half = detX_size // 2
+    padded_value_exact = int(np.sqrt(2 * (det_half**2))) - det_half
+    padded_add_margin = int(0.1 * padded_value_exact)
+    return padded_value_exact + padded_add_margin
